from colorama import Fore, Back, Style
from logcatcolor.column import *
import re
import StringIO

class Layout(object):
    MARKER_REGEX = re.compile(r"^--------- beginning of")
    MARKER_FORMAT = Fore.WHITE + Back.BLACK + Style.DIM + "%s" + Style.RESET_ALL
    def __init__(self, config, profile, width):
        self.columns = []
        self.config = config
        self.data = {}
        self.profile = profile
        self.width = width

        self.total_column_width = 0
        if self.COLUMNS:
            # first get the total column width, then construct each column
            for ColumnType in self.COLUMNS:
                self.total_column_width += config.get_column_width(ColumnType)

            for ColumnType in self.COLUMNS:
                column = ColumnType(self)
                self.columns.append(column)

        self.column_count = len(self.columns)

        if self.PATTERN:
            self.regex = re.compile(self.PATTERN)

    def match_marker(self, line):
        return self.MARKER_REGEX.match(line)

    def match_data(self, line):
        if not self.regex:
            return True

        match = self.regex.match(line)
        if not match:
            return False

        for name, value in match.groupdict().iteritems():
            self.data[name] = value.strip()
        return True

    def clear_data(self):
        self.data.clear()

    def include(self):
        if self.profile and not self.profile.include(self.data):
            return False

        return True

    def layout(self, line):
        if len(line) == 0:
            return None

        if self.match_marker(line):
            return self.layout_marker(line)

        if not self.match_data(line):
            return None

        if not self.include():
            self.clear_data()
            return None

        line = self.layout_columns(line)
        self.clear_data()
        return line

    def layout_marker(self, line):
        return self.MARKER_FORMAT % line

    def layout_columns(self, line):
        formatted = StringIO.StringIO()
        for index in range(0, self.column_count):
            column = self.columns[index]
            data = self.data[column.NAME]
            formatted.write(column.format(data))
            if index < self.column_count - 1:
                formatted.write(" ")

        return formatted.getvalue()

class RawLayout(Layout):
    NAME = "raw"
    def layout_columns(self, line):
       return line

class BriefLayout(Layout):
    "I/Tag(  PID): message"
    NAME = "brief"
    PRIORITY_PATTERN = r"(?P<priority>[A-Z])"
    PRIORITY_TAG_PATTERN = PRIORITY_PATTERN + r"/" + r"(?P<tag>.*?)"

    PID_PATTERN = r"(?P<pid>\d+)"
    PID_PAREN_PATTERN = r"\(\s*" + PID_PATTERN + r"\)"
    MESSAGE_PATTERN = r"(?P<message>.*?)"

    BRIEF_PATTERN = PRIORITY_TAG_PATTERN + \
              PID_PAREN_PATTERN + r": " + \
              MESSAGE_PATTERN
    PATTERN = r"^" + BRIEF_PATTERN + r"$"
    COLUMNS = (PIDColumn, TagColumn, PriorityColumn, MessageColumn)

class ProcessLayout(Layout):
    "I(  PID) message (Tag)"
    NAME = "process"
    PATTERN = r"^" + BriefLayout.PRIORITY_PATTERN + \
             BriefLayout.PID_PAREN_PATTERN + r" " + \
             BriefLayout.MESSAGE_PATTERN + r" " + \
             r"\((?P<tag>.+)\)$"
    COLUMNS = BriefLayout.COLUMNS

class TagLayout(Layout):
    "I/Tag  : message"
    NAME = "tag"
    PATTERN = r"^" + BriefLayout.PRIORITY_TAG_PATTERN + r": " + \
              BriefLayout.MESSAGE_PATTERN + r"$"
    COLUMNS = (TagColumn, PriorityColumn, MessageColumn)

class ThreadLayout(Layout):
    "I(  PID:TID) message"
    NAME = "thread"
    TID_HEX_PATTERN = r"(?P<tid>0x[0-9a-f]+)"
    PID_TID_HEX_PATTERN = BriefLayout.PID_PATTERN + r":" + TID_HEX_PATTERN

    PATTERN = r"^" + BriefLayout.PRIORITY_PATTERN + \
              r"\(\s*" + PID_TID_HEX_PATTERN + r"\) " + \
              BriefLayout.MESSAGE_PATTERN + r"$"
    COLUMNS = (PIDColumn, TIDColumn, PriorityColumn, MessageColumn)

class TimeLayout(Layout):
    "MM-DD HH:MM:SS.mmm D/Tag(  PID): message"
    NAME = "time"
    DATE_TIME_PATTERN = r"(?P<date>\d\d-\d\d)\s(?P<time>\d\d:\d\d:\d\d\.\d\d\d)"
    PATTERN = r"^" + DATE_TIME_PATTERN + r" " + BriefLayout.BRIEF_PATTERN + r"$"
    COLUMNS = (DateColumn, TimeColumn, ) + BriefLayout.COLUMNS

class ThreadTimeLayout(Layout):
    "MM-DD HH:MM:SS.mmm   PID   TID I ONCRPC  : rpc_handle_rpc_call: Find Status: 0 Xid: 7062"
    NAME = "threadtime"
    PATTERN = r"^" + TimeLayout.DATE_TIME_PATTERN + r"\s+" + \
              BriefLayout.PID_PATTERN + r"\s+" + \
              r"(?P<tid>\d+)\s+" + \
              BriefLayout.PRIORITY_PATTERN + r"\s+" + \
              r"(?P<tag>.*?)\s*: " + \
              BriefLayout.MESSAGE_PATTERN + r"$"
    COLUMNS = (DateColumn, TimeColumn, PIDColumn, TIDColumn, TagColumn, PriorityColumn, MessageColumn)

class LongLayout(Layout):
    "[ MM-DD HH:MM:SS.mmm   PID:TID I/Tag ]\nmessage"
    NAME = "long"
    PATTERN = r"^\[ " + TimeLayout.DATE_TIME_PATTERN + r"\s+" + \
                   ThreadLayout.PID_TID_HEX_PATTERN + r"\s+" + \
                   BriefLayout.PRIORITY_TAG_PATTERN + r"\s+\]$"
    COLUMNS = ThreadTimeLayout.COLUMNS

    def match_data(self, line):
        if not Layout.match_data(self, line):
            self.data["message"] = line

        return "message" in self.data and "tag" in self.data



import colorama
from colorama import Fore, Back, Style
import StringIO

colorama.init()

class Column(object):
    def __init__(self, layout):
        self.width = layout.config.get_column_width(self)

    def format(self, data):
        return self.FORMAT % data

class DateColumn(Column):
    NAME = "date"
    FORMAT = Fore.WHITE + Back.BLACK + Style.DIM + \
             "%s" + Style.RESET_ALL
    DEFAULT_WIDTH = 7

class TimeColumn(Column):
    NAME = "time"
    FORMAT = Fore.WHITE + Back.BLACK + Style.DIM + \
             "%s" + Style.RESET_ALL
    DEFAULT_WIDTH = 14

class PIDColumn(Column):
    NAME = "pid"
    DEFAULT_WIDTH = 8
    FORMAT = Fore.WHITE + Back.BLACK + Style.DIM + \
             "%s" + Style.RESET_ALL

    def format(self, pid):
        # center process info
        if self.width > 0:
            pid = pid.center(self.width)

        return Column.format(self, pid)

class TIDColumn(PIDColumn):
    NAME = "tid"

    def format(self, tid):
        # normalize thread IDs to be decimal
        if "0x" in tid:
            tid = str(int(tid, 16))

        return PIDColumn.format(self, tid)

class TagColumn(Column):
    NAME = "tag"
    DEFAULT_WIDTH = 20
    COLOR_NAMES = ("RED", "GREEN", "YELLOW", "BLUE", "MAGENTA", "CYAN", "WHITE")
    COLOR_MAP = {}

    @classmethod
    def init_color_map(cls):
        for color in cls.COLOR_NAMES:
            cls.COLOR_MAP[color] = getattr(Fore, color)

    def __init__(self, layout):
        Column.__init__(self, layout)

        tag_colors = None
        if layout.profile:
            tag_colors = layout.profile.get_tag_colors()

        self.tag_colors = tag_colors or {}
        self.last_used = self.COLOR_MAP.values()[:]

    # This will allocate a unique format for the given tag since we dont have
    # very many colors, we always keep track of the LRU
    def allocate_color(self, tag):
        if tag not in self.tag_colors:
            self.tag_colors[tag] = self.last_used[0]

        color = self.tag_colors[tag]
        self.last_used.remove(color)
        self.last_used.append(color)
        return color

    def format(self, tag):
        color = self.allocate_color(tag)
        if self.width > 2:
            if self.width < len(tag):
                tag = tag[0:self.width-2] + ".."

        tag = tag.rjust(self.width)
        return color + Style.DIM + tag + Style.RESET_ALL

TagColumn.init_color_map()

class PriorityColumn(Column):
    NAME = "priority"
    DEFAULT_WIDTH = 3
    COLORS = {
        "V": Fore.WHITE + Back.BLACK,
        "D": Fore.BLACK + Back.BLUE,
        "I": Fore.BLACK + Back.GREEN,
        "W": Fore.BLACK + Back.YELLOW,
        "E": Fore.BLACK + Back.RED,
        "F": Fore.BLACK + Back.RED
    }

    def __init__(self, layout):
        Column.__init__(self, layout)
        self.formats = {}
        for priority in self.COLORS.keys():
            self.formats[priority] = self.COLORS[priority] + \
                priority.center(self.width) + Style.RESET_ALL

    def format(self, priority):
        return self.formats[priority]

class MessageColumn(Column):
    NAME = "message"
    DEFAULT_WIDTH = 0
    def __init__(self, layout):
        self.width = None
        self.left = layout.total_column_width
        if layout.config.wrap and (not layout.profile or layout.profile.wrap):
            self.width = layout.width - self.left

    def format(self, message):
        # Don't wrap when width is None
        if not self.width:
            return message

        messagebuf = StringIO.StringIO()
        current = 0
        while current < len(message):
            next = min(current + self.width, len(message))
            messagebuf.write(message[current:next])
            if next < len(message):
                messagebuf.write("\n%s" % (" " * self.left))
            current = next
        return messagebuf.getvalue()



from logcatcolor.column import TagColumn
from logcatcolor.profile import Profile
import os
import platform
import re
import sys
import traceback

class LogcatColorConfig(object):
    DEFAULT_LAYOUT = "brief"
    DEFAULT_WRAP = True
    DEFAULT_ADB = None

    def __init__(self, options):
        self.options = options
        self.path = options.config or self.get_default_config()
        self.filters = {}

        self.config = { "Profile":  Profile }
        self.config.update(TagColumn.COLOR_MAP)

        if os.path.exists(self.path):
            # config file is just a python script that globals are imported from
            try:
                execfile(self.path, self.config)
            except:
                self.report_config_error()
                sys.exit(1)

            # clean up builtins and globals from self.config
            del self.config["__builtins__"]
            del self.config["Profile"]
            for color in TagColumn.COLOR_MAP.keys():
                del self.config[color]

        self.load_config()

    def report_config_error(self):
        config_error = """
########################################
# There was an error loading config from
# %(path)s
########################################

%(error)s"""

        print >>sys.stderr, config_error % {
            "path": self.path,
            "error": traceback.format_exc()
        }

    def get_default_config(self):
        env_key = "HOME"
        if platform.system() == "Windows":
            env_key = "USERPROFILE"

        home_dir = os.environ[env_key]
        return os.path.join(home_dir, ".logcat-color")

    def load_config(self):
        if self.options.wrap is None:
            self.wrap = True
            if "wrap" in self.config:
                self.wrap = self.config["wrap"]

    def get_default_layout(self):
        return self.config.get("default_layout", self.DEFAULT_LAYOUT)

    def get_column_width(self, column):
        return self.config.get(column.NAME + "_width", column.DEFAULT_WIDTH)

    def get_wrap(self):
        return self.config.get("wrap", self.DEFAULT_WRAP)

    def get_adb(self):
        return self.config.get("adb", self.DEFAULT_ADB)



#!/usr/bin/env python

import codecs
import json
from setuptools import setup, find_packages

setup_data = json.load(open("setup.json", "r"))

# real classy distutils, name / version have to be ascii encoded :(
for ascii_key in ("name", "version"):
    setup_data[ascii_key] = setup_data[ascii_key].encode("ascii")

setup_data["packages"] = find_packages()
setup(**setup_data)



import asyncore
import asynchat
import fcntl
import inspect
import logcatcolor.layout
import os
import sys
import traceback

# Parts copied from asyncore.file_dispatcher
class FileLineReader(asynchat.async_chat):
    LINE_TERMINATOR = "\n"

    def __init__(self, fd):
        asynchat.async_chat.__init__(self)
        self.connected = True
        self.log_buffer = []

        self.set_file(fd)
        self.set_terminator(self.LINE_TERMINATOR)

    def set_file(self, fd):
        try:
            # fd may be a file object
            fd = fd.fileno()
        except AttributeError:
            pass

        self.socket = asyncore.file_wrapper(fd)
        self._fileno = self.socket.fileno()
        self.add_channel()

        flags = fcntl.fcntl(fd, fcntl.F_GETFL, 0)
        flags = flags | os.O_NONBLOCK
        fcntl.fcntl(fd, fcntl.F_SETFL, flags)

    def collect_incoming_data(self, data):
        self.log_buffer.append(data)

    def found_terminator(self):
        line = "".join(self.log_buffer)
        try:
            self.process_line(line)
        except:
            traceback.print_exc()
            sys.exit(1)

        self.log_buffer = []

    def process_line(self):
        pass

class LogcatReader(FileLineReader):
    LAYOUTS = {}
    def __init__(self, file, config, profile=None, layout="brief", width=80):
        FileLineReader.__init__(self, file)
        LayoutType = self.LAYOUTS[layout]
        self.layout = LayoutType(config, profile, width)

    def process_line(self, line):
        formatted = self.layout.layout(line.strip())
        if formatted:
            print formatted

for member in inspect.getmembers(logcatcolor.layout):
    LayoutType = member[1]
    if LayoutType == logcatcolor.layout.Layout:
        continue
    if not inspect.isclass(LayoutType):
        continue
    if not logcatcolor.layout.Layout in LayoutType.__bases__:
        continue

    LogcatReader.LAYOUTS[LayoutType.NAME] = LayoutType



import re

class Profile(object):
    __profiles__ = {}

    @classmethod
    def get_profile(cls, name):
        return cls.__profiles__.get(name, None)

    def __init__(self, name=None, tags=None, priorities=None, filters=None,
            buffers=None, wrap=True, device=None, emulator=None, format=None):
        if not name:
            raise Exception("Profile is missing a name")

        self.name = name
        self.__profiles__[name] = self

        self.init_tags(tags)
        self.init_priorities(priorities)
        self.init_filters(filters)
        self.buffers = buffers
        self.wrap = wrap
        self.device = device
        self.emulator = emulator
        self.format = format

    def init_tags(self, tags):
        self.tags = None
        self.tag_colors = None
        if isinstance(tags, dict):
            self.tags = tags.keys()
            self.tag_colors = tags
        elif isinstance(tags, (list, tuple)):
            self.tags = tags
        elif tags:
            self.tags = (tags)

    def init_priorities(self, priorities):
        self.priorities = None
        if isinstance(priorities, (list, tuple)):
            self.priorities = priorities
        elif priorities:
            self.priorities = (priorities)

    def init_filters(self, filters):
        self.filters = []
        if not filters:
            return

        for filter in filters:
            if isinstance(filter, str):
                self.filters.append(self.regex_filter(filter))
            else:
                self.filters.append(filter)

    def regex_filter(self, regex):
        pattern = re.compile(regex)
        def __filter(data):
            if "message" not in data:
                return True
            return re.search(pattern, data["message"])
        return __filter

    def include(self, data):
        if self.tags and data["tag"] not in self.tags:
            return False

        if self.priorities and data["priority"] not in self.priorities:
            return False

        if self.filters:
            for filter in self.filters:
                if not filter(data):
                    return False

        return True

    def get_buffers(self):
        return self.buffers

    def get_tag_colors(self):
        return self.tag_colors
